//+------------------------------------------------------------------+ 
//|                                                     OnTheFly.mq5 |
//|            Copyright 2012, https://login.mql5.com/ru/users/tol64 |
//|                                        http://tol64.blogspot.com |
//+------------------------------------------------------------------+  
#property copyright   "Copyright 2012, http://tol64.blogspot.com"
#property link        "http://tol64.blogspot.com"
#property description "email: hello.tol64@gmail.com"
#property version     "1.00"
//---
#define szArrIP 5 // Size of the parameter array
#define NAME_EXPERT MQL5InfoString(MQL5_PROGRAM_NAME) // Name of EA
#define TRM_DP TerminalInfoString(TERMINAL_DATA_PATH) // Folder that contains the terminal data
//+------------------------------------------------------------------+ 
//| STANDARD LIBRARIES                                               |
//+------------------------------------------------------------------+ 
#include <Trade/SymbolInfo.mqh>
#include <Trade/Trade.mqh>
//+------------------------------------------------------------------+ 
//| CUSTOM LIBRARIES                                                 |
//+------------------------------------------------------------------+ 
#include "Include/!OnChartEvent.mqh"
#include "Include/CREATE_PANEL.mqh"
#include "Include/FILE_OPERATIONS.mqh"
#include "Include/ERRORS.mqh"
#include "Include/ARRAYS.mqh"
#include "Include/TRADE_SIGNALS.mqh"
#include "Include/TRADE_FUNCTIONS.mqh"
#include "Include/GET_STRING.mqh"
#include "Include/GET_COLOR.mqh"
#include "Include/ADD_FUNCTIONS.mqh"
//+------------------------------------------------------------------+ 
//| CREATING CLASS INSTANCES                                         |
//+------------------------------------------------------------------+ 
CSymbolInfo mysymbol; // CSymbolInfo class object
CTrade      mytrade;  // CTrade class object
//+------------------------------------------------------------------+ 
//| EXTERNAL PARAMETERS                                              |
//+------------------------------------------------------------------+ 
input int    Period_Ind      = 10;    // Indicator Period
input double TakeProfit      = 100;   // Take Profit (p)
input double StopLoss        = 30;    // Stop Loss (p)
input bool   Reverse         = false; // Reverse Position
input double Lot             = 0.1;   // Lot
//---
input string slash="";       // * * * * * * * * * * * * * * * * * * *
sinput bool  InfoPanel       = true;  // On/Off Info Panel
sinput bool  SettingOnTheFly = false; // "On The Fly" Setting
//+------------------------------------------------------------------+ 
//| GLOBAL VARIABLES                                                 |
//+------------------------------------------------------------------+ 
int hdlSI=INVALID_HANDLE; // Signal indicator handle
double lcheck=0; // for the check of parameter values
bool isPos=false; // Position availability
//--- COPY OF EXTERNAL PARAMETERS
int    gPeriod_Ind = 0;
double gTakeProfit = 0.0;
double gStopLoss   = 0.0;
bool   gReverse    = false;
double gLot        = 0.0;
//+------------------------------------------------------------------+ 
//| EXPERT ADVISOR INITIALIZATION                                    |
//+------------------------------------------------------------------+ 
void OnInit()
  {
   if(NotTest()) { EventSetTimer(1); } // Set the timer
//---
   Init_arr_vparams(); // Initialization of the array of parameter values
   SetParameters(); // Set the parameters
   GetIndicatorsHandles(); // Get indicator handles
   NewBar(); // New bar initialization
   SetInfoPanel(); // Info panel
  }
//+------------------------------------------------------------------+ 
//| CURRENT SYMBOL TICKS                                             |
//+------------------------------------------------------------------+ 
void OnTick()
  {
// If the bar is not new, exit
   if(!NewBar()) { return; }
   else
     { TradingBlock(); }
  }
//+------------------------------------------------------------------+ 
//| TIMER                                                            |
//+------------------------------------------------------------------+ 
void OnTimer()
  {
   SetParameters(); SetInfoPanel();
  }
//+------------------------------------------------------------------+ 
//| EXPERT ADVISOR DEINITIALIZATION                                  |
//+------------------------------------------------------------------+ 
void OnDeinit(const int reason)
  {
// Get the deinitialization reason code
   if(NotTest())
     { Print(getUnitReasonText(reason)); }
//---
// When deleting from the chart
   if(reason==REASON_REMOVE)
     {
      // Delete all objects created by the Expert Advisor
      DeleteAllExpertObjects();
      //---
      if(NotTest()) { EventKillTimer(); } // Stop the timer
      IndicatorRelease(hdlSI); // Delete the indicator handle
     }
  }
//+------------------------------------------------------------------+ 
//| GETTING SIGNAL INDICATOR HANDLES                                 |
//+------------------------------------------------------------------+ 
void GetIndicatorsHandles()
  {
   hdlSI=iCustom(_Symbol,_Period,"JMA.ex5",gPeriod_Ind,0);
//---
   if(hdlSI!=INVALID_HANDLE)
     { Print(__FUNCTION__,": Handle of the signal indicator ("+IS(hdlSI)+") for "+_Symbol+" obtained"); }
   else
     { Print(__FUNCTION__,": Failed to get the signal indicator handle for "+_Symbol+""); }
  }
//+------------------------------------------------------------------+ 
//| CHECKING NEW BAR                                                 |
//+------------------------------------------------------------------+ 
bool NewBar()
  {
   static datetime NewBar=NULL;
   static datetime lastbar_time[1]={0};
//---
// Time of the current bar
   if(CopyTime(_Symbol,_Period,0,1,lastbar_time)==-1)
     { Print(__FUNCTION__,": Error when copying the bar opening time: "+IS(Error())+""); }
//---
   if(NewBar==0) // If this is the first function call
     {
      NewBar=lastbar_time[0]; // Set the time and exit 
      Print(__FUNCTION__,": Initialization ["+_Symbol+"][TF: "+TFtoS(_Period)+"]["+TSdms(lastbar_time[0])+"]");
      return(false);
     }
//---
// If the time is different
   if(NewBar!=lastbar_time[0])
     {
      NewBar=lastbar_time[0]; // Set the time and exit 
      return(true); // Store the time and return true
     }
//---
   return(false); // If we reached this line, it means that the bar is not new, so return false
  }
//+------------------------------------------------------------------+ 
//| SETTING PARAMETERS IN TWO MODES                                  |
//+------------------------------------------------------------------+ 
// If this variable is set to false, the parameters in the file are read from the array,
// where they are saved for quick access after they have been read for the first time.
// The variable is zeroed out upon the change in the value on the panel.
bool flgRead=false;
double arrParamIP[]; // Array where the parameters from the file are saved
//---
void SetParameters()
  {
// If currently in the tester or
// in real time but with the "On The Fly" Setting mode disabled
   if(!NotTest() || (NotTest() && !SettingOnTheFly))
     {
      // Zero out the variable and parameter array
      flgRead=false;
      ArrayResize(arrParamIP,0);
      //---
      // Check the Indicator Period for correctness
      if(Period_Ind<=0)
        { lcheck=10; }
      else { lcheck=Period_Ind; }
      gPeriod_Ind=(int)lcheck;
      //---
      gStopLoss=StopLoss;
      gTakeProfit=TakeProfit;
      gReverse=Reverse;
      //---
      // Check the Lot for correctness
      if(Lot<=0)
        { lcheck=0.1; }
      else { lcheck=Lot; }
      gLot=lcheck;
     }
   else // If "On The Fly" Setting mode is enabled
     {
      // Check if there is a file for writing/reading parameters to/from the file
      string lpath="";
      //---
      // If the folder exists
      if((lpath=CheckCreateGetPath())!="")
        {
         // Write or read the file
         WriteReadParameters(lpath);
        }
     }
  }
//+------------------------------------------------------------------+ 
//| ZEROING OUT VARIABLES                                            |
//+------------------------------------------------------------------+ 
void iZeroMemory()
  {
   flgDialogWin=false;
   ArrayResize(lenum,0);
   ArrayResize(lenmObj,0);
   number=-1; flgRead=false;
  }
//+------------------------------------------------------------------+ 
